from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.resource_range import ResourceRange


T = TypeVar("T", bound="ResourceOption")


@_attrs_define
class ResourceOption:
    """
    Attributes:
        cpu (ResourceRange):
        memory (ResourceRange):
    """

    cpu: "ResourceRange"
    memory: "ResourceRange"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cpu = self.cpu.to_dict()

        memory = self.memory.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "cpu": cpu,
                "memory": memory,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.resource_range import ResourceRange

        d = src_dict.copy()
        cpu = ResourceRange.from_dict(d.pop("cpu"))

        memory = ResourceRange.from_dict(d.pop("memory"))

        resource_option = cls(
            cpu=cpu,
            memory=memory,
        )

        resource_option.additional_properties = d
        return resource_option

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
